% copyright by Silvia Gandy, Tokyo Institute of Technology
% June, 2011

function [Idx, B, XX, nXX, SMALLSCALE] = setup_problem_random_tensors(n, ...
                                            r, percentage_available) 
%  setup_problem_random_tensors: this functions generates a random problem
%  instance of a tensor completion problem

fprintf('\n--------setting up a randomly generated problem instance----------\n\n');

orderX = size(n,2);

if prod(n)/n(1) < 100*100 
    SMALLSCALE = 1;
    %fprintf('small dimension problem, using a full svd decomposition\n');
else
    SMALLSCALE = 0;
end

m = percentage_available *prod(n);  %number of observed entries 

fprintf('Problem dimensions:\n');
array_format = '%i';
for i = 2:orderX
    array_format = [array_format, ', %i'];
end
fprintf(['tensor size:  n = [ ', array_format, ']\n'], n);
fprintf('number of known entries: m = %1.1f*prod(n)\n', m/prod(n));
fprintf('rank of the unfoldings of the generated tensor: r = %i\n', r);

%% setup problem (construct B and XX, s.t. P_Idx(XX) = B)
%construct XX, a 3-way tensor which contains our input matrices
%we are considering a tensor completion setup, i.e., B contains entries of XX

SUPERSYMMETRIC = 0;

if SUPERSYMMETRIC 
    G = zeros(r*ones(1,orderX));
    for i =1:r
        G(i*ones(1,orderX)) = 1;
    end
else
    G = randn(r*ones(1,orderX));
end

% construct XX as XX = G \times_1 U1 \times_2 U2 \times_3 U3
dimG = r*ones(1,orderX);
for i=1:orderX
    U = randn(n(i), r);
    dimG(i) = n(i);
    G = refold_mode_n_Nthorder(U*unfold_mode_n_Nthorder(G, i), i, dimG);
end
XX = G;
XX = XX/max(XX(:));
nXX = norm(XX(:));

% tensor completion
Idx = randsample(prod(n), m);%index set of the sampled entries

B = XX(Idx);  %sampled entries

end