% copyright by Silvia Gandy, Tokyo Institute of Technology
% June, 2011

%run this script to setup a random problem instance of a tensor completion
%problem and call different algorithms to solve this instance

addpath ../PROPACK; %add path leading to the folder containing the implementation of lansvd

%define identifiers for the algorithms
DOUGLAS_RACHFORD = 1; 
EXACT_ADM = 2;

list_algorithms = [EXACT_ADM]; %each setting is solved with all of the listed algorithms
%list_algorithms = [EXACT_ADM];

%caseIDs = 1:4;
caseIDs = 4; %which settings shall be executed? cases defined below
nrRuns = size(caseIDs, 2);  %how often shall each setting be applied?

nrAlgs = size(list_algorithms,2);

%initialize output structure 
results.algorithms = list_algorithms;
results.elapsedTime = zeros(nrAlgs, nrRuns);
results.relError = zeros(nrAlgs, nrRuns);
results.iter = zeros(nrAlgs, nrRuns);
results.percentage_available = zeros(1, nrRuns);
results.r = zeros(1,nrRuns);
results.n = zeros(nrRuns, 5);

% HERE YOU CAN CHOOSE WHICH SETTING YOU WANT TO RUN
% SET itnr TO THE APPROPRIATE SETTING
for itnr = caseIDs
    
    %% parameters to setup the problem
    switch itnr
        case 1
            n = [20 20 20 20 20];
            percentage_available = 0.2;
            r = 2;               
        case 2
            n = [20 20 20 20 20];
            percentage_available = 0.3;
            r = 2; 
        case 3
            n = [50 50 50 50];
            percentage_available = 0.4;
            r = 4;            
        case 4
            n = [20 30 40];
            percentage_available = 0.6;
            r = 2;    
    end
    
    results.percentage_available(itnr) = percentage_available;
    results.r(itnr) = r;
    results.n(itnr, 1:size(n,2)) = n;
    
    %% setup problem
    [Idx, B, XX, nXX, SMALLSCALE] = setup_problem_random_tensors(n, r, percentage_available);
    
    nrank_XX = multilinear_rank_tensor(XX);
    fprintf('checking the n-rank of the generated tensor XX: '); display(nrank_XX);
    %VERBOSE = 1;     %calculate the rel. error in each step 
    VERBOSE = 0;      %minimal output
    
    %% call algorithm
    nrRun = 0;
    for alg_id = list_algorithms
            nrRun = nrRun +1;
            maxiter = 2000;  %maximal number of iterations   
            tic
            
            switch alg_id
                case DOUGLAS_RACHFORD
                    [X_out, nrIter] = tensor_solve_DR_final(n, Idx, B, maxiter, ...
                                   VERBOSE, XX, nXX);
                case EXACT_ADM
                    [X_out, Y, nrIter] = tensor_solve_exactADM_final(n, Idx, B, maxiter, ...
                                   VERBOSE, XX, nXX);
            end;
                               
            elapsed_time = toc
            
            %evaluate result and save in file
            res = norm(XX(:) - X_out(:));
            rel_err_k = norm(res)/nXX;
            fprintf('Algorithm ended with a relative error (norm(Xk-XX)/norm(XX)) of: %4g\n', rel_err_k);
            results.elapsedTime(nrRun, itnr) = elapsed_time;
            results.relError(nrRun, itnr) = rel_err_k;
            results.iter(nrRun, itnr) = nrIter;
            
            %save(filename, 'results');
            
            
    end
    
end

filename = ['values_algorithmComparison_', datestr(now,31), '.mat'];
save(filename, 'results');
