%% Demo for generating 3D exponential signals
%% 
%% Reference:
%% [1] Jiaxi Ying, Hengfa Lu, Qingtao Wei, Jian-Feng Cai, Di Guo, Jihui Wu, Zhong Chen, Xiaobo Qu. Hankel matrix nuclear norm regularized tensor completion for N-dimensional exponential signals, IEEE Transactions on Signal Processing, 65(14): 3702-3717, 2017
%% Available: http://dx.doi.org/10.1109/TSP.2017.2695566   or   http://csrc.xmu.edu.cn/pdf/2017_TSP_TensorHankelNMR.pdf
%% [2] Xiaobo Qu, Maxim Mayzel, Jian-Feng Cai, Zhong Chen, Vladislav Orekhov. Accelerated NMR spectroscopy with low-rank reconstruction, Angewandte Chemie International Edition, 54(3):852-854, 2015. 
%% Available: http://dx.doi.org/10.1002/anie.201409291
%%
%% By Jiaxi Ying, Xiaobo Qu
%% Computational Sensing Group at Xiamen University   http://www.quxiaobo.org 
%% Email: quxiaobo <at> xmu.edu.cn
%% 2017-05-08
%%
clear;
close all;
clc;
%%
path(path,'Toolbox_Plot3D_Decomposition')
%%
row=50;                                                    % the ambient dimensions of the signal 50*50*50
col=50;          
ver=50;          
NumPeaks=50;                                               % the number of exponentials R
%% set rand seed
ctime = datestr(now,30);
tseed = str2num(ctime((end-5):end));
rand('seed',tseed);
%% generate a 3D simulated exponential signal (undamped or damped)

% generate undamped signal
% signal = generate3Dsignal(NumPeaks,row,col,ver); % If you want to obtain reconstructions of undamped signal, please uncomment this line

% generate damped signal
signal = generate_3Ddamping_signal(NumPeaks,row,col,ver); % generate a 3D simulated damped exponential signal with size "row*col*ver" 

spectra3D=fft3c(signal);  
%% draw the spectrum
figure,voxel3((abs(spectra3D)),'fast',false);title('The original spectra')